package javax.management.openmbean;

import java.util.Arrays;
import java.util.Collections;
import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.SortedMap;
import java.io.Serializable;

public class CompositeDataSupport implements CompositeData, Serializable
{
    static final long serialVersionUID = 8003518976613702244L;
    private final SortedMap<String, Object> contents;
    private final CompositeType compositeType;
    
    public CompositeDataSupport(CompositeType compositeType, String[] array, Object[] array2) throws OpenDataException {
        this(makeMap(array, array2), compositeType);
    }
    
    private static SortedMap<String, Object> makeMap(String[] array, Object[] array2) throws OpenDataException {
        if (array == null || array2 == null) {
            throw new IllegalArgumentException("Null itemNames or itemValues");
        }
        if (array.length == 0 || array2.length == 0) {
            throw new IllegalArgumentException("Empty itemNames or itemValues");
        }
        if (array.length != array2.length) {
            throw new IllegalArgumentException("Different lengths: itemNames[" + array.length + "], itemValues[" + array2.length + "]");
        }
        TreeMap<Object, Object> treeMap = new TreeMap();
        int i = 0;
        while (i < array.length) {
            String str = array[i];
            if (str == null || str.equals("")) {
                throw new IllegalArgumentException("Null or empty item name");
            }
            if (treeMap.containsKey(str)) {
                throw new OpenDataException("Duplicate item name " + str);
            }
            treeMap.put(array[i], array2[i]);
            ++i;
        }
        return treeMap;
    }
    
    public CompositeDataSupport(CompositeType compositeType, Map<String, ?> map) throws OpenDataException {
        this(makeMap(map), compositeType);
    }
    
    private static SortedMap<String, Object> makeMap(Map<String, ?> map) {
        if (map == null || map.isEmpty()) {
            throw new IllegalArgumentException("Null or empty items map");
        }
        TreeMap<String, Object> treeMap = new TreeMap();
        for (Object next : map.keySet()) {
            if (next == null || next.equals("")) {
                throw new IllegalArgumentException("Null or empty item name");
            }
            if (!(next instanceof String)) {
                throw new ArrayStoreException("Item name is not string: " + next);
            }
            treeMap.put((String)next, map.get(next));
        }
        return treeMap;
    }
    
    private CompositeDataSupport(SortedMap<String, Object> contents, CompositeType compositeType) throws OpenDataException {
        if (compositeType == null) {
            throw new IllegalArgumentException("Argument compositeType cannot be null.");
        }
        Set<String> keySet = compositeType.keySet();
        Set<Object> keySet2 = contents.keySet();
        if (!keySet.equals(keySet2)) {
            TreeSet<Object> obj = new TreeSet(keySet);
            obj.removeAll(keySet2);
            TreeSet<Object> obj2 = new TreeSet(keySet2);
            obj2.removeAll(keySet);
            if (!obj.isEmpty() || !obj2.isEmpty()) {
                throw new OpenDataException("Item names do not match CompositeType: names in items but not in CompositeType: " + obj2 + "; names in CompositeType but not in items: " + obj);
            }
        }
        for (String str : keySet) {
            Object value = contents.get(str);
            if (value != null) {
                OpenType<Object> type = compositeType.getType(str);
                if (!type.isValue(value)) {
                    throw new OpenDataException("Argument value of wrong type for item " + str + ": value " + value + ", type " + type);
                }
                continue;
            }
            else {
                continue;
            }
        }
        this.compositeType = compositeType;
        this.contents = contents;
        return;
    }
    
    @Override
    public CompositeType getCompositeType() {
        return this.compositeType;
    }
    
    @Override
    public Object get(String s) {
        if (s == null || s.trim().equals("")) {
            throw new IllegalArgumentException("Argument key cannot be a null or empty String.");
        }
        if (!this.contents.containsKey(s.trim())) {
            throw new InvalidKeyException("Argument key=\"" + s.trim() + "\" is not an existing item name for this CompositeData instance.");
        }
        return this.contents.get(s.trim());
    }
    
    @Override
    public Object[] getAll(String[] array) {
        if (array == null || array.length == 0) {
            return new Object[0];
        }
        Object[] array2 = new Object[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = this.get(array[i]);
        }
        return array2;
    }
    
    @Override
    public boolean containsKey(String s) {
        return s != null && !s.trim().equals("") && this.contents.containsKey(s);
    }
    
    @Override
    public boolean containsValue(Object o) {
        return this.contents.containsValue(o);
    }
    
    @Override
    public Collection<?> values() {
        return Collections.unmodifiableCollection(this.contents.values());
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CompositeData)) {
            return false;
        }
        CompositeData compositeData = (CompositeData)o;
        if (!this.getCompositeType().equals(compositeData.getCompositeType())) {
            return false;
        }
        if (this.contents.size() != compositeData.values().size()) {
            return false;
        }
        for (Map.Entry entry : this.contents.entrySet()) {
            Object value = entry.getValue();
            Object value2 = compositeData.get((String)entry.getKey());
            if (value == value2) {
                continue;
            }
            if (value == null) {
                return false;
            }
            if (!(value.getClass().isArray() ? Arrays.deepEquals(new Object[] { value }, new Object[] { value2 }) : value.equals(value2))) {
                return false;
            }
            continue;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int hashCode = this.compositeType.hashCode();
        for (Object next : this.contents.values()) {
            if ((next instanceof Object[])) {
                hashCode += Arrays.deepHashCode((Object[])(Object[])next);
            }
            else if ((next instanceof byte[])) {
                hashCode += Arrays.hashCode((byte[])(byte[])next);
            }
            else if ((next instanceof short[])) {
                hashCode += Arrays.hashCode((short[])(short[])next);
            }
            else if ((next instanceof int[])) {
                hashCode += Arrays.hashCode((int[])(int[])next);
            }
            else if ((next instanceof long[])) {
                hashCode += Arrays.hashCode((long[])(long[])next);
            }
            else if ((next instanceof char[])) {
                hashCode += Arrays.hashCode((char[])(char[])next);
            }
            else if ((next instanceof float[])) {
                hashCode += Arrays.hashCode((float[])(float[])next);
            }
            else if ((next instanceof double[])) {
                hashCode += Arrays.hashCode((double[])(double[])next);
            }
            else if ((next instanceof boolean[])) {
                hashCode += Arrays.hashCode((boolean[])(boolean[])next);
            }
            else if (next != null) {
                hashCode += next.hashCode();
            }
            else {
                continue;
            }
        }
        return hashCode;
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "(compositeType=" + this.compositeType.toString() + ",contents=" + this.contentString() + ")";
    }
    
    private String contentString() {
        StringBuilder sb = new StringBuilder("{");
        String str = "";
        for (Map.Entry entry : this.contents.entrySet()) {
            sb.append(str).append((String)entry.getKey()).append("=");
            String deepToString = Arrays.deepToString(new Object[] { entry.getValue() });
            sb.append(deepToString.substring(1, deepToString.length() - 1));
            str = ", ";
        }
        sb.append("}");
        return sb.toString();
    }
}
